# -*- coding: utf-8 -*-
"""

    ファイル名：   PilatusUtils.py

    処理内容：

       Pilatus 画像データ関連の共通処理

"""
from __future__ import division, print_function, unicode_literals
    # /   を使うと商は float
    # //  を使うと商は int

import os
import sys
import glob
import re
import pandas as pd
import PilatusCounter
from DebugQueue             import debug_queue_get

separator_line  = re.compile( '^\s*$' )
prefix_re       = re.compile( '^#\s+Pilatus1 fileprefix:\s+(\w+)' )
num_cycles_re   = re.compile( '^#\s+Pilatus1 Number of cycles:\s+(\d+)' )
num_changes_re  = re.compile( '^#\s+Pilatus1 Detector position:\s+Change\s+(\d+)' )
position_re     = re.compile( '^#\s+Pilatus1 Detector position(\(original\)|\d+):\s+([+-]?\d+\.?\d*),([+-]?\d+\.?\d*)' )
options_re      = re.compile( '^#\s+User options:' )
tiff_file_re    = re.compile( '^(([^_]+)_(\d+)_(\d+))\.tif' )   # TODO: ignore case
                                #   sid    cno   mno

def get_in_folder_info( in_folder ):
    orig_folder = os.getcwd()
    os.chdir( in_folder )
    log_files = glob.glob( '*.log' )
    mask_files = glob.glob( '*.mask' )
    os.chdir( orig_folder )

    log_file, mask_file = (None, None )
    if len( log_files ) >= 1:
        log_file = log_files[0]

    mask_file = None
    if len( mask_files ) >= 1:
        mask_file = mask_files[0]

    return log_file, mask_file

def get_data_info( in_folder, adj_folder, syn_folder, pilatus_counter, counter_id,
                    sample_complete=False, for_test_data=False ):

    image_data_info = []
    log_file        = None
    mask_file       = None

    if not in_folder or not os.path.exists( in_folder ):
        return log_file, mask_file, [], pilatus_counter

    orig_folder = os.getcwd()

    log_file, mask_file = get_in_folder_info( in_folder )

    # TODO: refactor to move PilatusCounter out.
    if not pilatus_counter:
        pilatus_counter = PilatusCounter.Counter( in_folder )

    counter_dict = pilatus_counter.get_counter_dict( counter_id )
    # print( 'counter_dict=', counter_dict )

    org_file_dict = {}
    if in_folder and os.path.exists( in_folder ):
        os.chdir( in_folder )
        for file in glob.glob( '*.tif' ):
            fkey = re.sub( r'_\d+\.tif$', '', file, flags=re.I )
            # print( 'fkey=', fkey )
            org_file_dict[ fkey ] = file

    adj_file_dict = {}
    if adj_folder and os.path.exists( adj_folder ):
        os.chdir( adj_folder )
        for file in glob.glob( '*.tif' ):
            fkey = re.sub( r'_[^_]+\.tif$', '', file, flags=re.I )
            adj_file_dict[ fkey ] = file

    syn_file_dict = {}
    if syn_folder and os.path.exists( syn_folder ):
        os.chdir( syn_folder )
        for file in glob.glob( '*.tif' ):
            fkey = re.sub( r'_[^_]+\.tif$', '', file, flags=re.I )
            syn_file_dict[ fkey ] = file

    os.chdir( orig_folder )

    prefix = ''
    measurement_text = ''
    text_dict = {}

    if debug_queue_get() == __name__ + '.die()': die()  # die(): undefined function

    fh = open( in_folder + '/' + log_file )

    fkey_type_is_old = True
    fkey_type_is_new = False

    for line in fh.readlines():
        # print line

        if separator_line.match( line ):
            if prefix != '':
                text_dict[ prefix ] = measurement_text
            measurement_text = ''
        else:
            measurement_text += line

        m0 = prefix_re.match( line )
        if m0:
            # print line
            prefix = m0.group( 1 )
            # print prefix
            # pos_array = [ prefix ]

            pos_info_counter = 0
            num_cycles  = 0
            num_changes = 0
            pos_array = []
            continue

        m1 = num_cycles_re.match( line )
        if m1:
            num_cycles = int( m1.group( 1 ) )

        m2 = num_changes_re.match( line )
        if m2:
            # print line
            num_changes = int( m2.group( 1 ) )
            # print num_changes
            continue

        m3 = position_re.match( line )
        if m3:
            position = m3.group( 1 )
            x = m3.group( 2 )
            y = m3.group( 3 )
            # print "\tposition_re.match: %s\t%s,%s" % (position, x, y)
            pos_array.append( [ x, y ] )
            continue

        m4 = options_re.match( line )
        if m4:
            if fkey_type_is_old:
                info_array = []
                for i in range( num_changes ):
                    fkey = '%s_%d' % ( prefix, i )
                    if i == 0:
                        orig_file_count = counter_dict.get( fkey, 0 )
                        if orig_file_count == 0:
                            fkey_type_is_old = False
                            break
                        else:
                            ratio = 1.0
                    else:
                        if counter_id == 'None':
                            ratio = 1.0
                        else:
                            if orig_file_count > 0:
                                ratio = counter_dict.get( fkey ) / orig_file_count
                            else:
                                ratio = None
                    syn_file = syn_file_dict.get( prefix )
                    if not syn_file:
                        # prefix_syn.tif がなければ prefix_{i}_syn.tif を設定する。
                        syn_file = syn_file_dict.get( fkey )
                    info_array.append( [ org_file_dict.get( fkey ), pos_array[i], ratio, adj_file_dict.get( fkey ), syn_file ] )

                if len( info_array ):
                    if not sample_complete or len( info_array ) == num_changes:
                        # すなわち、 sample_complete == True の場合は num_changes に見たなければ
                        # 採集しない。
                        image_data_info.append( [ prefix, info_array ] )

            if fkey_type_is_new or not fkey_type_is_old:
                # print( 'num_cycles=', num_cycles, 'counter_id=', counter_id )
                for j in range( num_cycles ):
                    new_prefix_ = '%s_%d' % ( prefix, j )
                    info_array = []
                    for k in range( num_changes ):
                        fkey = '%s_d%d' % ( new_prefix_, k )
                        # print( 'fkey=', fkey )
                        if k == 0:
                            orig_file_count = counter_dict.get( fkey, 0 )
                            if orig_file_count == 0:
                                # print( 'no count for fkey=', fkey )
                                break
                            else:
                                fkey_type_is_new = True
                                ratio = 1.0
                        else:
                            if counter_id == 'None':
                                ratio = 1.0
                            else:
                                if orig_file_count > 0:
                                    ratio = counter_dict.get( fkey ) / orig_file_count
                                    # print( 'ratio=', ratio )
                                else:
                                    assert( False )
                        syn_file = syn_file_dict.get( new_prefix_ )
                        if not syn_file:
                            # TODO: 必要？
                            pass
                        info_array.append( [ org_file_dict.get( fkey ), pos_array[k], ratio, adj_file_dict.get( fkey ), syn_file ] )

                    if len( info_array ):
                        if not sample_complete or len( info_array ) == num_changes:
                            # すなわち、 sample_complete == True の場合は num_changes に見たなければ
                            # 採集しない。
                            image_data_info.append( [ new_prefix_, info_array ] )

    if measurement_text != '':
        text_dict[ prefix ] = measurement_text

    fh.close()

    # print( 'image_data_info[ 0 ]=', image_data_info[ 0 ] )

    data_array = make_data_array( image_data_info )

    if for_test_data:
        return data_array, text_dict
    else:
        return log_file, mask_file, data_array, pilatus_counter

def make_data_array( image_data_info ):
    data_array = []

    for prefix, info_rec in image_data_info:

        if info_rec[0][0] == None:
            # 基準画像がないデータは無視する。
            continue

        # print( prefix, info_rec )
        data_array.append( [ prefix, info_rec ] )
        # self.data_array は、表示された Table オブジェクトのセル（表示値）と
        # 単純な対応関係を持つように構築するが、
        # その平坦な構造上、prefix ごとのグルーピングが失われるため、
        # i によってそのグルーピングを把握できるようにしておく。
        """
                [ AgBh002,      [ AgBh002_0_00000.tif,  ['0.70000','-0.35000'], 1.0, None, None ],
                                [ AgBh002_1_00000.tif,  ['5', '3'],             1.0, AgBh002_1_adj.tif, AgBh002_1_syn.tif ],
                                [ AgBh002_2_00000.tif,  ['-5', '-3'],           1.0, AgBh002_2_adj.tif, AgBh002_2_syn.tif ],
                ]
        """

    return data_array
